from datetime import datetime

# Helper functions
def generate_prompt(message, instructions):
    return "\n".join([message, instructions])

def quit_option(text):
    return "\n".join([text, ("(Or to stop this process entirely, "
        + "just type `quit`.)")])

def reaction_instructions(error=False):
    start = "Fix this and r" if error else "R"
    verb = "retry" if error else "proceed"
    return (
        "%seact to this message with :+1: to %s, or :-1: to quit." 
        % (start, verb)
    )

# All other message functions
def aws_bucket_fail():
    return "One or more of your AWS bucket names is incorrect."

def aws_connect_fail():
    return "Cannot connect to AWS."

def aws_file_transfer(file, bucket, delete=False):
    verb = "Deleting" if delete else "Uploading"
    preposition = "from" if delete else "to"
    return (
        "%s `%s` %s Amazon S3 bucket `%s`" % (verb, file, preposition, bucket)
    )

def aws_invalidation_fail(file):
    return (
        (
            "Your cache invalidation request for `%s` failed. "
            + "Ensure the file you wish to invalidate "
            + "actually exists in CloudFront."
        ) % file
    )

def aws_invalidation_sent(file):
    return (
        (
            "Your cache invalidation request for `%s` sent successfully. "
            + "Expect it to complete within the next 15 minutes."
        ) % file
    )

def aws_transfer_fail(file, delete=False):
    return ("The file `%s` failed to %s." 
        % (file, "delete" if delete else "upload"))

def aws_transfer_success():
    return "Success!"

def bot_available():
    return (
        "QA-Bot is available to run. To see how you can use the QA-Bot, "
        + "type `@qa-bot help`."
    )

def bot_parse_fail():
    return (
        "You seem like you want to speak with me, but I don't understand "
        + "your intentions. Type `@qa-bot help` to learn how to do better."
    )

def bot_running(app_name):
    return ":loading: QA-Bot is running a `%s` process now. :loading:" % app_name

def bot_unavailable():
    return "QA-Bot is currently unavailable."

def bad_slack_creds():
    return ("Your Slack credentials file contains invalid JSON. Please fix and restart.")

def cannot_find_file_location(file_location):
    return quit_option(
        (
            "Sorry, we cannot connect to a path matching the "
            + "value `%s`. Try again."
        ) % file_location
    )

def close_slack_loop():
    return "\nClosing Slack contact loop at %s." % str(datetime.now())

def credentials_fail():
    return generate_prompt(
        (
            "Sorry, your credentials and/or worksheet "
            + "are not working as expected."
        ), 
        reaction_instructions(error=True)
    )

def db_connect_fail(instance):
    return (
        (
            "Cannot connect to instance `%s`. Verify your credentials " 
            + "are correct."
        ) % instance
    )

def dm_suggestion():
    return "Let's continue this conversation in private."

def entity_not_found(uuid):
    return (
        (
            "Sorry, there appears not to be a published entity with uuid "
            + "`%s` in Postgres. You'll need to start over."
        ) % uuid
    )

def generic_fail(content):
    return "Something went wrong. Detail: `%s`" % content

def goodbye():
    return "Goodbye!"

def help_intro():
    return "Oh, do you not know how this works?"

def ignoring_data(db_name):
    return "Never mind; ignoring %s data." % db_name

def improper_credentials(slack=False):
    text = ("Sorry, you do not have all the proper keys in %s." 
        % ("slack.json" if slack else "variables.json"))
    return text if slack else generate_prompt(
        text, 
        reaction_instructions(error=True)
    )

def invalid_json():
    return generate_prompt(
        "Your variables file contains invalid JSON.", 
        reaction_instructions(error=True)
    )

def need_photos(photos_present):
    message = ("You are missing the image file(s) listed below in "
        + "your specified directory:")
    for key, value in photos_present.items():
        if value == False:
            message = "\n".join([message, key])
    return generate_prompt(message, reaction_instructions(error=True))

def new_slack_loop():
    return ("\nBeginning new loop to listen for contact from Slack at %s." 
        % str(datetime.now()))

def notify_public():
    return "Notifying public chat."

def nuclear_fail():
    return (
        "Sorry; nuclear mode appears to be disarmed at the moment. "
        + "Try another mode."
    )

def nuclear_match_fail():
    return (
        "Sorry, your nuclear code did not match the required sequence. "
        + "You have failed to launch. Try again later."
    )

def nuclear_prompt():
    return (
        "You have selected nuclear mode. "
        + "This mode only requires approval of a single user -- you."
    )

def nuclear_reaction_instructions():
    return (
        "React to this message with your four-emoji nuclear code to "
        + "proceed. Failure to respond with the proper emoji in the proper "
        + "order will lead to cancellation."
    )

def photos_present():
    return "All photos present!"

def print_code(code):
    print("\nWelcome to nuclear emoji mode. Your launch code is:"),
    for emoji in code:
        print(":%s:" % emoji),
    print("\n")

def quit_run():
    return "Cancelling bot actions for this entity; moving on!"

def redis_connect_fail(instance):
    return (
        (
            "Cannot connect to redis instance `%s`. Verify your " 
            + "connection string is connect."
        ) % instance
    )


def request_image_location():
    return quit_option(
        "Please give me the location of your required image files. "
        + "Note: the location *MUST* be a folder on a shared drive."
    )

def slack_connect_fail():
    return "Could not connect to Slack."

def slack_creds_missing():
    return ("You're missing your required Slack credentials file. Get it in place and restart.")

def update_instructions():
    return "Do you wish to overwrite? (:+1: or :-1:)"

def uuid_instructions():
    return (
        "Reply in a thread to this message with a comma-separated list "
        + "of the uuids of each entity you wish to work with."
    )
def validating_slack_creds():
    return "\nValidating Slack credentials JSON."

def writing_to_db(db_name, delete=False):
    verb = "Removing" if delete else "Inserting"
    preposition = "from" if delete else "into"
    return "%s rows %s `%s` db." % (verb, preposition, db_name)