import psycopg2, psycopg2.extras
from messages import db_connect_fail, entity_not_found, generic_fail
from queries import entity_select_statement
from slack import send_thread_message

def db_connect(vars, db_name, ping=False):
    """ Connect to database and return connection and cursor """
    try:
        conn_string = (
            "host='%s' dbname='%s' user='%s' password='%s'" 
            % (
                vars["host"], 
                db_name, 
                vars["user"], 
                vars["pass"]
                )
        )
        conn = psycopg2.connect(conn_string)
        cur = conn.cursor()
    except Exception, e:
        return db_connect_fail(vars["host"])
    if ping:
        db_disconnect(cur,conn)
        return None
    else:
        return conn, cur

def db_disconnect(cur, conn):
    """ Close cursor and database connection. """
    cur.close()
    conn.close()

def get_entity_metadata(
    cred, 
    uuid, 
    client, 
    channel, 
    thread, 
    db_name, 
    row=None
):
    """ Return basic data for a given entity. """
    conn, cur = db_connect(cred, db_name)
    try:
        cur.execute(entity_select_statement(cur, uuid))
        row = cur.fetchone()
    except psycopg2.DataError, e:
        send_thread_message(
            client=client, 
            contact_channel=channel, 
            contact_thread=thread, 
            text=entity_not_found(uuid)
        )
        return None, None, None, False
    except Exception, e:
        send_thread_message(
            client=client, 
            contact_channel=channel, 
            contact_thread=thread, 
            text=generic_fail(e)
        )
    db_disconnect(cur, conn)
    if row is not None:
        return row[0], row[1], row[2], False
    send_thread_message(
        client=client, 
        contact_channel=channel, 
        contact_thread=thread, 
        text=entity_not_found(uuid)
    )
    return None, None, None, True