import emoji
import flags
import messages
import photos
import postgres
import queries
import redis_data
import slack
import validate

import sys
import unittest
from StringIO import StringIO

class TestEmoji(unittest.TestCase):
    def setUp(self):
        self.emoji_array = ["test1","test2","test3"]
        self.code_length = 2

    def test_emoji_code_is_proper_length(self):
        emoji_code = emoji.generate_emoji_code(
            self.emoji_array, 
            self.code_length
        )
        self.assertEqual(len(emoji_code), self.code_length)

    def test_emoji_code_only_contains_initial_array_elements(self):
        emoji_code = emoji.generate_emoji_code(
            self.emoji_array[:], 
            self.code_length
        )
        for e in emoji_code:
            self.assertTrue(e in self.emoji_array)

class TestFlags(unittest.TestCase):
    def test_string_not_matching_bot_name_returns_none(self):
        value = bot_communication("test", "failure")
        self.assertFalse(bot_communication("test", "failure"))

    def test_string_only_has_bot_name_returns_none(self):
        value = bot_communication("test", "<@test>")
        self.assertFalse(bot_communication("test", "<@test>"))

    def test_string_with_bot_name_not_first_returns_none(self):
        value = bot_communication("test", "hey <@test>")
        self.assertFalse(bot_communication("test", "<@test>"))

    def test_string_starting_with_valid_bot_name_returns_command(self):
        self.assertEqual(
            bot_communication("test", "<@test> command"), 
            ["command"]
        )

class TestValidate(unittest.TestCase):
    def test_goodbye_should_close_app(self):
        with self.assertRaises(SystemExit):
            validate.goodbye()

    def test_slack_key_fail_should_terminate_app(self):
        with self.assertRaises(SystemExit):
            validate.key_fail(is_slack=True)

    def test_non_slack_key_fail_should_not_terminate_app(self):
        try:
            validate.key_fail(is_slack=False)
        except ExceptionType:
            self.fail("key_fail() raised ExceptionType unexpectedly!")

    def test_non_slack_key_fail_returns_correct_message(self):
        self.assertEqual(
            validate.key_fail(), 
            (
                "Sorry, you do not have all the proper keys in variables.json."
                + "\nFix this and react to this message with :+1: to retry, "
                + "or :-1: to quit."
            )
        )

    def test_invalid_slack_root_path_causes_app_termination(self):
        with self.assertRaises(SystemExit):
            validate.slack(root_path = 'Test')



'''

def slack(root_path, creds=None):
    if not os.path.isfile(os.path.join(root_path, "env","slack.json")):
        print(messages.slack_creds_missing())
        goodbye()
    print(messages.validating_slack_creds())
    with open(os.path.join(root_path,"env","slack.json")) as slack_creds:
        try:
            creds = json.load(slack_creds)
        except ValueError, e:
            print (messages.bad_slack_creds())
            goodbye()
    if not all(key in ["token", "channel", "bot_id", "users"] 
        for key in creds.keys()):
        key_fail(True)
    return creds

'''

if __name__ == '__main__':
    unittest.main()