import json
import messages
import os
from StringIO import StringIO
import sys

def goodbye():
    """ Quit the app """
    print(messages.goodbye())
    return sys.exit()

def key_fail(is_slack=False):
    """ Notify user of improper credentials. If credentials in 
    question are Slack credentials, shut app down 
    """
    if is_slack:
        print(messages.improper_credentials(is_slack))
        goodbye()
    return messages.improper_credentials(is_slack)

def return_validated_json(creds_blob):
    """ Validate json for Slack credentials """
    print(messages.validating_slack_creds())
    try:
        return json.load(creds_blob)
    except ValueError, e:
        print (messages.bad_slack_creds())
        goodbye()
    
def check_for_required_keys(creds):
    """ Ensure Slack credential json has all required keys """
    required_keys = ["token", "channel", "bot_id", "users"]
    if not all(key in required_keys for key in creds.keys()):
        key_fail(is_slack=True)

def slack(root_path, creds=None):
    """ Retrieve, validate, and return credentials for Slack connection """
    if not os.path.isfile(os.path.join(root_path, "env","slack.json")):
        print(messages.slack_creds_missing())
        goodbye()
    with open(os.path.join(root_path,"env","slack.json")) as slack_json:
        creds = return_validated_json(slack_json)

    check_for_required_keys(creds)
    return creds